package de.unisaar.jfap.ex2;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicInteger;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Actor with associated movies.
 *
 * @author Jörg Steffen, DFKI
 */
@JsonIdentityInfo(
    generator = ObjectIdGenerators.PropertyGenerator.class,
    property = "id",
    scope = Actor.class)
// alternative if there is no field that can be used as id:
// @JsonIdentityInfo(generator = ObjectIdGenerators.IntSequenceGenerator.class, property = "@id")
public class Actor implements Serializable {

  private static final Logger logger = LoggerFactory.getLogger(Actor.class);

  private static final long serialVersionUID = 3873319516877846163L;

  // generate integer ids
  private static AtomicInteger idGenerator = new AtomicInteger();

  private int id;
  private String firstName;
  private String lastName;
  private List<Movie> movies;

  public Actor() {
    // default constructor; required for Jackson deserialization
  }

  /**
   * Create a new actor with the given parameters.
   *
   * @param firstName the first name
   * @param lastName the last name
   */
  public Actor(String firstName, String lastName) {
    this.id = idGenerator.getAndIncrement();
    this.firstName = firstName;
    this.lastName = lastName;
    this.movies = new ArrayList<>();
  }

  public int getId() {
    return this.id;
  }

  public void setId(int id) {
    this.id = id;
  }

  public String getFirstName() {
    return this.firstName;
  }

  public void setFirstName(String firstName) {
    this.firstName = firstName;
  }

  public String getLastName() {
    return this.lastName;
  }

  public void setLastName(String lastName) {
    this.lastName = lastName;
  }

  public List<Movie> getMovies() {
    return this.movies;
  }

  public void setMovies(List<Movie> movies) {
    this.movies = movies;
  }

  /**
   * Add given movie to the movies of this actor. Also add this actor to the actors of the given
   * movie.
   *
   * @param mov the movie to add
   */
  public void addMovie(Movie mov) {
    if (!this.movies.contains(mov)) {
      logger.debug("adding {} to {}", this, mov);
      this.movies.add(mov);
      mov.getActors().add(this);
    }
  }

  @Override
  public int hashCode() {
    return Objects.hash(this.id);
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    Actor other = (Actor) obj;
    return this.id == other.id;
  }

  @Override
  public String toString() {
    return String.format("%s %s", this.firstName, this.lastName).strip();
  }
}
