package de.unisaar.jfap.ex2;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicInteger;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Movie with associated actors.
 *
 * @author Jörg Steffen, DFKI
 */
@JsonIdentityInfo(
    generator = ObjectIdGenerators.PropertyGenerator.class,
    property = "id",
    scope = Movie.class)
// alternative if there is no field that can be used as id:
// @JsonIdentityInfo(generator = ObjectIdGenerators.IntSequenceGenerator.class, property = "@id")
public class Movie implements Serializable {

  private static final Logger logger = LoggerFactory.getLogger(Movie.class);

  private static final long serialVersionUID = -1768187784846022713L;

  // generate integer ids
  private static AtomicInteger idGenerator = new AtomicInteger();

  private int id;
  private String title;
  private int year;
  private List<Actor> actors;

  public Movie() {
    // default constructor, required for Jackson deserialization
  }

  /**
   * Create a new movie with the given parameters.
   *
   * @param title the movie's title
   * @param year the movie's release year
   */
  public Movie(String title, int year) {
    this.id = idGenerator.getAndIncrement();
    this.title = title;
    this.year = year;
    this.actors = new ArrayList<>();
  }

  public int getId() {
    return this.id;
  }

  public void setId(int id) {
    this.id = id;
  }

  public String getTitle() {
    return this.title;
  }

  public void setTitle(String title) {
    this.title = title;
  }

  public int getYear() {
    return this.year;
  }

  public void setYear(int year) {
    this.year = year;
  }

  public List<Actor> getActors() {
    return this.actors;
  }

  public void setActors(List<Actor> actors) {
    this.actors = actors;
  }

  /**
   * Add given actor to the actors of this movie. Also add this movie to the movies of the given
   * actor.
   *
   * @param act the actor to add
   */
  public void addActor(Actor act) {
    if (!this.actors.contains(act)) {
      logger.debug("adding {} to {}", act, this);
      this.actors.add(act);
      act.getMovies().add(this);
    }
  }

  @Override
  public int hashCode() {
    return Objects.hash(this.id);
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    Movie other = (Movie) obj;
    return this.id == other.id;
  }

  @Override
  public String toString() {
    return this.title;
  }
}
